@@ -4,7 +4,7 @@
  *	  Utility and convenience functions for fmgr functions that return
  *	  sets and/or composite types, or deal with VARIADIC inputs.
  *
- * Copyright (c) 2002-2018, PostgreSQL Global Development Group
+ * Copyright (c) 2002-2019, PostgreSQL Global Development Group
  *
  * IDENTIFICATION
  *	  src/backend/utils/fmgr/funcapi.c
@@ -14,6 +14,7 @@
 #include "postgres.h"
 
 #include "access/htup_details.h"
+#include "access/relation.h"
 #include "catalog/namespace.h"
 #include "catalog/pg_proc.h"
 #include "catalog/pg_type.h"
@@ -38,16 +39,16 @@ typedef struct polymorphic_actuals
 
 static void shutdown_MultiFuncCall(Datum arg);
 static TypeFuncClass internal_get_result_type(Oid funcid,
-						 Node *call_expr,
-						 ReturnSetInfo *rsinfo,
-						 Oid *resultTypeId,
-						 TupleDesc *resultTupleDesc);
+											  Node *call_expr,
+											  ReturnSetInfo *rsinfo,
+											  Oid *resultTypeId,
+											  TupleDesc *resultTupleDesc);
 static void resolve_anyelement_from_others(polymorphic_actuals *actuals);
 static void resolve_anyarray_from_others(polymorphic_actuals *actuals);
 static void resolve_anyrange_from_others(polymorphic_actuals *actuals);
 static bool resolve_polymorphic_tupdesc(TupleDesc tupdesc,
-							oidvector *declared_args,
-							Node *call_expr);
+										oidvector *declared_args,
+										Node *call_expr);
 static TypeFuncClass get_type_func_class(Oid typid, Oid *base_typeid);
 
 
@@ -97,7 +98,6 @@ init_MultiFuncCall(PG_FUNCTION_ARGS)
 		 */
 		retval->call_cntr = 0;
 		retval->max_calls = 0;
-		retval->slot = NULL;
 		retval->user_fctx = NULL;
 		retval->attinmeta = NULL;
 		retval->tuple_desc = NULL;
@@ -138,21 +138,6 @@ per_MultiFuncCall(PG_FUNCTION_ARGS)
 {
 	FuncCallContext *retval = (FuncCallContext *) fcinfo->flinfo->fn_extra;
 
-	/*
-	 * Clear the TupleTableSlot, if present.  This is for safety's sake: the
-	 * Slot will be in a long-lived context (it better be, if the
-	 * FuncCallContext is pointing to it), but in most usage patterns the
-	 * tuples stored in it will be in the function's per-tuple context. So at
-	 * the beginning of each call, the Slot will hold a dangling pointer to an
-	 * already-recycled tuple.  We clear it out here.
-	 *
-	 * Note: use of retval->slot is obsolete as of 8.0, and we expect that it
-	 * will always be NULL.  This is just here for backwards compatibility in
-	 * case someone creates a slot anyway.
-	 */
-	if (retval->slot != NULL)
-		ExecClearTuple(retval->slot);
-
 	return retval;
 }
 
@@ -251,72 +236,6 @@ get_expr_result_type(Node *expr,
 										  NULL,
 										  resultTypeId,
 										  resultTupleDesc);
-	else if (expr && IsA(expr, RowExpr) &&
-			 ((RowExpr *) expr)->row_typeid == RECORDOID)
-	{
-		/* We can resolve the record type by generating the tupdesc directly */
-		RowExpr    *rexpr = (RowExpr *) expr;
-		TupleDesc	tupdesc;
-		AttrNumber	i = 1;
-		ListCell   *lcc,
-				   *lcn;
-
-		tupdesc = CreateTemplateTupleDesc(list_length(rexpr->args), false);
-		Assert(list_length(rexpr->args) == list_length(rexpr->colnames));
-		forboth(lcc, rexpr->args, lcn, rexpr->colnames)
-		{
-			Node	   *col = (Node *) lfirst(lcc);
-			char	   *colname = strVal(lfirst(lcn));
-
-			TupleDescInitEntry(tupdesc, i,
-							   colname,
-							   exprType(col),
-							   exprTypmod(col),
-							   0);
-			TupleDescInitEntryCollation(tupdesc, i,
-										exprCollation(col));
-			i++;
-		}
-		if (resultTypeId)
-			*resultTypeId = rexpr->row_typeid;
-		if (resultTupleDesc)
-			*resultTupleDesc = BlessTupleDesc(tupdesc);
-		return TYPEFUNC_COMPOSITE;
-	}
-	else if (expr && IsA(expr, Const) &&
-			 ((Const *) expr)->consttype == RECORDOID &&
-			 !((Const *) expr)->constisnull)
-	{
-		/*
-		 * When EXPLAIN'ing some queries with SEARCH/CYCLE clauses, we may
-		 * need to resolve field names of a RECORD-type Const.  The datum
-		 * should contain a typmod that will tell us that.
-		 */
-		HeapTupleHeader rec;
-		Oid			tupType;
-		int32		tupTypmod;
-
-		rec = DatumGetHeapTupleHeader(((Const *) expr)->constvalue);
-		tupType = HeapTupleHeaderGetTypeId(rec);
-		tupTypmod = HeapTupleHeaderGetTypMod(rec);
-		if (resultTypeId)
-			*resultTypeId = tupType;
-		if (tupType != RECORDOID || tupTypmod >= 0)
-		{
-			/* Should be able to look it up */
-			if (resultTupleDesc)
-				*resultTupleDesc = lookup_rowtype_tupdesc_copy(tupType,
-															   tupTypmod);
-			return TYPEFUNC_COMPOSITE;
-		}
-		else
-		{
-			/* This shouldn't really happen ... */
-			if (resultTupleDesc)
-				*resultTupleDesc = NULL;
-			return TYPEFUNC_RECORD;
-		}
-	}
 	else
 	{
 		/* handle as a generic expression; no chance to resolve RECORD */
@@ -1053,9 +972,7 @@ get_func_arg_info(HeapTuple procTup,
 /*
  * get_func_trftypes
  *
- * Returns the number of transformed types used by the function.
- * If there are any, a palloc'd array of the type OIDs is returned
- * into *p_trftypes.
+ * Returns the number of transformed types used by function.
  */
 int
 get_func_trftypes(HeapTuple procTup,
@@ -1084,6 +1001,7 @@ get_func_trftypes(HeapTuple procTup,
 			ARR_HASNULL(arr) ||
 			ARR_ELEMTYPE(arr) != OIDOID)
 			elog(ERROR, "protrftypes is not a 1-D Oid array");
+		Assert(nelems >= ((Form_pg_proc) GETSTRUCT(procTup))->pronargs);
 		*p_trftypes = (Oid *) palloc(nelems * sizeof(Oid));
 		memcpy(*p_trftypes, ARR_DATA_PTR(arr),
 			   nelems * sizeof(Oid));
@@ -1437,7 +1355,7 @@ build_function_result_tupdesc_d(char prokind,
 	if (numoutargs < 2 && prokind != PROKIND_PROCEDURE)
 		return NULL;
 
-	desc = CreateTemplateTupleDesc(numoutargs, false);
+	desc = CreateTemplateTupleDesc(numoutargs);
 	for (i = 0; i < numoutargs; i++)
 	{
 		TupleDescInitEntry(desc, i + 1,
@@ -1557,7 +1475,7 @@ TypeGetTupleDesc(Oid typeoid, List *colaliases)
 		/* OK, get the column alias */
 		attname = strVal(linitial(colaliases));
 
-		tupdesc = CreateTemplateTupleDesc(1, false);
+		tupdesc = CreateTemplateTupleDesc(1);
 		TupleDescInitEntry(tupdesc,
 						   (AttrNumber) 1,
 						   attname,
