@@ -11,7 +11,7 @@
  *		as this notice is not removed.
  *
  *	The author is not responsible for loss or damages that may
- *	result from it's use.
+ *	result from its use.
  *
  *
  * IDENTIFICATION
@@ -58,20 +58,6 @@ typedef enum _teSection
 	SECTION_POST_DATA			/* stuff to be processed after data */
 } teSection;
 
-/* Parameters needed by ConnectDatabase; same for dump and restore */
-typedef struct _connParams
-{
-	/* These fields record the actual command line parameters */
-	char	   *dbname;			/* this may be a connstring! */
-	char	   *pgport;
-	char	   *pghost;
-	char	   *username;
-	trivalue	promptPassword;
-	/* If not NULL, this overrides the dbname obtained from command line */
-	/* (but *only* the DB name, not anything else in the connstring) */
-	char	   *override_dbname;
-} ConnParams;
-
 typedef struct _restoreOptions
 {
 	int			createDB;		/* Issue commands to create the database */
@@ -121,9 +107,12 @@ typedef struct _restoreOptions
 	SimpleStringList tableNames;
 
 	int			useDB;
-	ConnParams	cparams;		/* parameters to use if useDB */
-
+	char	   *dbname;			/* subject to expand_dbname */
+	char	   *pgport;
+	char	   *pghost;
+	char	   *username;
 	int			noDataForFailedTables;
+	trivalue	promptPassword;
 	int			exit_on_error;
 	int			compression;
 	int			suppressDumpWarnings;	/* Suppress output of WARNING entries
@@ -138,9 +127,10 @@ typedef struct _restoreOptions
 
 typedef struct _dumpOptions
 {
-	ConnParams	cparams;
-
-	bool		oids;
+	const char *dbname;			/* subject to expand_dbname */
+	const char *pghost;
+	const char *pgport;
+	const char *username;
 
 	int			binary_upgrade;
 
@@ -150,10 +140,10 @@ typedef struct _dumpOptions
 	int			dumpSections;	/* bitmask of chosen sections */
 	bool		aclsSkip;
 	const char *lockWaitTimeout;
+	int			dump_inserts;	/* 0 = COPY, otherwise rows per INSERT */
 
 	/* flags for various command-line long options */
 	int			disable_dollar_quoting;
-	int			dump_inserts;
 	int			column_inserts;
 	int			if_exists;
 	int			no_comments;
@@ -181,6 +171,7 @@ typedef struct _dumpOptions
 	char	   *outputSuperuser;
 
 	int			sequence_data;	/* dump sequence data even in schema-only mode */
+	int			do_nothing;
 } DumpOptions;
 
 /*
@@ -257,24 +248,15 @@ typedef void (*SetupWorkerPtrType) (Archive *AH);
  * Main archiver interface.
  */
 
-extern void ConnectDatabase(Archive *AHX,
-							const ConnParams *cparams,
-							bool isReconnect);
+extern void ConnectDatabase(Archive *AH,
+							const char *dbname,
+							const char *pghost,
+							const char *pgport,
+							const char *username,
+							trivalue prompt_password);
 extern void DisconnectDatabase(Archive *AHX);
 extern PGconn *GetConnection(Archive *AHX);
 
-/* Called to add a TOC entry */
-extern void ArchiveEntry(Archive *AHX,
-			 CatalogId catalogId, DumpId dumpId,
-			 const char *tag,
-			 const char *namespace, const char *tablespace,
-			 const char *owner, bool withOids,
-			 const char *desc, teSection section,
-			 const char *defn,
-			 const char *dropStmt, const char *copyStmt,
-			 const DumpId *deps, int nDeps,
-			 DataDumperPtr dumpFn, void *dumpArg);
-
 /* Called to write *data* to the archive */
 extern void WriteData(Archive *AH, const void *data, size_t dLen);
 
@@ -294,8 +276,8 @@ extern Archive *OpenArchive(const char *FileSpec, const ArchiveFormat fmt);
 
 /* Create a new archive */
 extern Archive *CreateArchive(const char *FileSpec, const ArchiveFormat fmt,
-			  const int compression, bool dosync, ArchiveMode mode,
-			  SetupWorkerPtrType setupDumpWorker);
+							  const int compression, bool dosync, ArchiveMode mode,
+							  SetupWorkerPtrType setupDumpWorker);
 
 /* The --list option */
 extern void PrintTOCSummary(Archive *AH);
